﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Problems.BinPacking.Interfaces;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Common;
using HeuristicLab.Problems.BinPacking.Shapes;

namespace HeuristicLab.Problems.BinPacking.Dimensions {
  [Item("Occupated Points", "A datastructure holding all currently occupied points of a packing-bin.")]
  [StorableClass]
  public abstract class OccupiedPoints<D, B, I> : Item
    where D : class, IPackingDimensions
    where B : PackingShape<D>, IPackingBin
    where I : PackingShape<D>, IPackingItem {

    public OccupiedPoints(B binMeasures) : base() { }
    [StorableConstructor]
    protected OccupiedPoints(bool deserializing) : base(deserializing) { }
    protected OccupiedPoints(OccupiedPoints<D,B,I> original, Cloner cloner)
      : base(original, cloner) {
    }


    public abstract bool IsPositionFeasible(I measures, D position);
    public abstract void OccupyPoints(I measures, D position, int itemID);
    public abstract bool IsPointOccupied(D point);
    public abstract int ShortestPossibleSideFromEP(D ep);
    public abstract bool IsStaticStable(I item, D ep);
    public abstract bool WeightIsSupported(I item, D ep, ItemList<I> items);
    public abstract void ChangeBinMeasures(B binMeasures);
  }
}
